@@ -1,5 +1,6 @@
+import io
 from dataclasses import dataclass, field
-from typing import Any, List
+from typing import List, TextIO
 
 
 class ParseError(Exception):
@@ -29,13 +30,18 @@ class ConfigFile:
         except KeyError:
             return False
 
-    # TODO file-like type
-    def build(self, buf: Any) -> None:
+    def build(self, buf: TextIO) -> None:
         buf.write(self.initial_comment)
         for s in self.sections:
             s.build(buf)
         buf.write(self.final_comment)
 
+    @property
+    def text(self) -> str:
+        buf = io.StringIO()
+        self.build(buf)
+        return buf.getvalue()
+
     def set_value(self, section: str, key: str, value: str) -> None:
         try:
             s = self[section]
@@ -62,8 +68,7 @@ class ConfigSection:
     newline: str
     entries: List["ConfigEntry"] = field(default_factory=list)
 
-    # TODO file-like type
-    def build(self, buf: Any) -> None:
+    def build(self, buf: TextIO) -> None:
         buf.write(
             self.leading_whitespace
             + self.leading_square_bracket
@@ -99,12 +104,24 @@ class ConfigSection:
                 whitespace_before_text="  " if i > 0 else "",
                 whitespace_after_text="",
             )
-            for i, line in enumerate(value.splitlines(False))
+            for i, line in enumerate(value.splitlines(False) or [""])
         ]
 
         for e in self.entries:
             if e.key.lower() == key:
+                had_value = e.value and bool(e.value[0].text)
+
                 e.value = valuelines
+                if e.whitespace_before_value and not valuelines[0].text:
+                    # Now has a trailing space, remove
+                    e.whitespace_before_value = ""
+                elif (
+                    not e.whitespace_before_value
+                    and not had_value
+                    and valuelines[0].text
+                ):
+                    # Add it back
+                    e.whitespace_before_value = " "
                 break
         else:
             self.entries.append(
@@ -113,7 +130,7 @@ class ConfigSection:
                     equals="=",
                     value=valuelines,
                     whitespace_before_equals=" ",
-                    whitespace_before_value=" ",
+                    whitespace_before_value=" " if valuelines[0].text else "",
                 )
             )
 
@@ -137,8 +154,7 @@ class ConfigEntry:
             ]
         )
 
-    # TODO
-    def build(self, buf: Any) -> None:
+    def build(self, buf: TextIO) -> None:
         buf.write(
             self.whitespace_before_key
             + self.key
@@ -158,8 +174,7 @@ class ValueLine:
     whitespace_after_text: str
     newline: str
 
-    # TODO
-    def build(self, buf: Any) -> None:
+    def build(self, buf: TextIO) -> None:
         buf.write(
             self.whitespace_before_text
             + self.text
